/* XVCL - the xview classes

   XVCL might one day be a hopefully useful class library
   for xview programming in C++. Just now, it slowly grows
   as I write programs, so many many things are missing.

   Copyright (C) 1998 Frank Rustemeyer.
   This file is part of the Emperor package.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston,
   MA 02111-1307, USA.

   If you have changes and you want them to be included in a future
   release of Emperor, please e-mail them to mambres@mud.tap.de.

   This file declares the classes for some generic data types. 

   98/11/30, Frank Rustemeyer <mambres@mud.tap.de>: Released.
*/

#ifndef _DATAH 
#define _DATAH

#define DELETELIST(x) while(x.root) x.remove(x.root)

#include <stdio.h>  /* for the data type FILE */

class coordinate
{   public:
       int x;
       int y;
       void set(int sx, int sy) { x = sx; y = sy; };
       coordinate operator + (const coordinate &c);
       int operator - (coordinate &c); // distance between two points
       int operator == (coordinate &c) const;
    };

class list
  // a linked list. members will be added by giving the list
  // as an argument to the listmember constructor
  // the list also has functions for being displayed in the main window 
{   public:
        friend class listmember;
	list(void);
	int init(class listmember * member);
	  // initialises list when member is the first member to be added
	  // returns zero if there are already members in the list
	void remove(class listmember * member);
	  // remove and destruct a member
	int length(void) const { return len; };
	  // returns the length of the list
	int position(void) const { return pos; };
	  // returns the position of the root pointer
	void save(FILE *f);
	  // save list.
	void load(FILE *f);
	  // load list. This needs to be defined by the application itself
	class listmember * root;
	  // pointer to listmember at position pos.
    protected:
	int before(class listmember * member) const;
	  // checks if member appears in the list before root
	int len;  // current length of the list
	int pos;  // current position of root
    };

class listmember  // generic member of a list (abstract base class)
{   public:
	listmember(list & thislist, listmember *aft = NULL);
	  // create a member in thislist (after member aft, or at the end)
	listmember(void);                  // create member outside list
	virtual ~listmember(void) {};
	  // delete a member. should be called ONLY from list::remove
	  // to make sure list::length is adjusted
        void unlink(void);
        void addtolist(list & thislist, listmember *aft = NULL);
        // insert this listmember into thislist (after member aft, or at the end)
	listmember * previous(void) const { return prv; };
	  // the previous list member
	listmember * next(void) const { return nxt; };
	  // the next list member
	listmember * first(void) const;    // the first member in the list
	listmember * last(void) const;     // the last member in the list
	virtual void save(FILE *f) = 0;
	  // save this listmember to a file, called by list::save
	virtual void load(char *s) = 0;
	  // load this listmember from a string read from a file,
          // called by list::load
    private:
	listmember * prv;  // pointer to previous listmember
	listmember * nxt;  // pointer to next listmember
    };

#endif
